import pytest

from poetry_analysis import rhyme_detection as rt


def test_find_last_stressed_syllable():
    """Check that the last syllable with a stress marker
    higher than 0 in a syllable sequence is the first syllable
    in the output sequence."""
    syllables = [
        "H AA1 R",
        "D UU1",
        "H OE1",
        "RT EE1 N",
        "M EE2 G",
        "T IH0",
        "S T EH2",
        "M AX0",
    ]
    # when
    result = rt.find_last_stressed_syllable(syllables)
    # then
    assert result == ["S T EH2", "M AX0"]



from poetry_analysis.lyrical_subject import detect_lyrical_subject


def test_detects_explicit_subject():
    poem_text = "Jeg er en poet"
    assert detect_lyrical_subject(poem_text)["explicit_subject"] is True


def test_detects_explicit_object():
    poem_text = "Du sa det til meg."
    assert detect_lyrical_subject(poem_text)["explicit_object"] is True


def test_detects_implicit():
    poem_text = "Vi leser bøker."
    assert detect_lyrical_subject(poem_text)["implicit"] is True


def test_detects_deixis():
    poem_text = "I går var en fin dag."
    assert detect_lyrical_subject(poem_text)["deixis"] is True



import pytest

from poetry_analysis.rhyme_detection import longest_common_substring


@pytest.mark.parametrize(
    "sequence1, sequence2",
    [
        ("klangen", "klang"),
        ("klang", "klangen"),
        ("arbeider", "arbeidene"),
        ("klangen", "sang"),
        ("sang", "klangen"),
        (["F", "R", "YY1", "D"], ["S", "YY1", "D"]),
        (["S", "II1", "N"], ["D", "II2", "N"]),
        ("F R YY D", "S YY D"),
        ("S II N", "D II N"),
        ("G UH L", "J UH L"),
        ("G UH2 L", "F UH2 L"),
        ("B OO D", "S T OO D"),
    ],
)
def test_any_common_substring_is_found(sequence1, sequence2):
    result = longest_common_substring(sequence1, sequence2)
    assert result



import json
import logging
import re
import string
from dataclasses import dataclass
from pathlib import Path
from typing import Generator

import numpy as np
from convert_pa import convert_nofabet
from nb_tokenizer import tokenize

from poetry_analysis import utils


@dataclass
class Poem:
    _id: str
    text: str
    stanzas: list


@dataclass
class Verse:
    id_: str | int
    rhyme_score: int = 0
    rhyme_tag: str = ""
    text: str = ""
    transcription: str = ""
    tokens: list | None = None
    syllables: list | None = None
    last_token: str | None = None
    rhymes_with: str | int | None = None

    @property
    def dict(self) -> dict:
        """Return the Verse object as a dictionary."""
        dictionary = self.__dict__
        dictionary["verse_id"] = self.id_
        del dictionary["id_"]
        return dictionary


def is_stressed(syllable: str | list) -> bool:
    """Check if a syllable is stressed by searching for stress markers.

    Stress markers:
        0 - Vowel/syllable nucleus without stress
        1 - Primary stress with toneme 1
        2 - Primary stress with toneme 2
        3 - Secondary stress
    """
    if isinstance(syllable, list):
        syllable = " ".join(syllable)
    result = re.search(r"[123]", syllable)
    return bool(result)


def strip_stress(phoneme: str) -> str:
    """Strip the stress marker from a phoneme."""
    return phoneme.strip("0123")


def is_nucleus(symbol: str, orthographic: bool = False) -> bool:
    """Check if a phoneme or a letter is a valid syllable nucleus."""
    if orthographic:
        valid_nuclei = utils.VALID_NUCLEI
    else:
        valid_nuclei = convert_nofabet.PHONES_NOFABET.get("nuclei")
    return strip_stress(symbol) in valid_nuclei


def find_nucleus(word: str, orthographic: bool = False) -> re.Match:
    """Check if a word has a valid syllable nucleus."""
    if orthographic:
        valid_nuclei = utils.VALID_NUCLEI
    else:
        valid_nuclei = convert_nofabet.PHONES_NOFABET.get("nuclei")
    rgx = re.compile(rf"({'|'.join(valid_nuclei)})")
    return rgx.search(word)


def remove_syllable_onset(syllable: list) -> list:
    """Split a syllable nucleus and coda from the onset to find the rhyming part of the syllable."""
    for idx, phone in enumerate(syllable):
        if is_nucleus(phone):
            return syllable[idx:]
    logging.debug("No nucleus found in %s", syllable)


def do_syll_seqs_rhyme(syll1: list, syll2: list):
    """Check  if each syllable in two syllable sequences are identical, apart from the stress marker."""
    if all(strip_stress(s1) == strip_stress(s2) for s1, s2 in zip(syll1, syll2)):
        return True
    return False


def score_rhyme(sequence1: str, sequence2: str, orthographic: bool = False) -> float:
    """Check if two words rhyme and return a rhyming score.

    1:      Only the syllable nucleus + coda (=rhyme) match # perfect or proper rhyme
    0.5:    NØDRIM or lame rhyme. One of the words is fully contained in the other, e.g. 'tusenfryd' / 'fryd'
    0:      No match
    """

    substring = shared_ending_substring(sequence1, sequence2)

    if not substring:
        logging.debug(
            "No shared ending substring found in %s and %s", sequence1, sequence2
        )
        return 0

    nucleus = find_nucleus(substring, orthographic=orthographic)

    if not nucleus:
        logging.debug("no nucleus found in %s", substring)
        return 0
    if utils.is_grammatical_suffix(substring):
        logging.debug("only the grammatical suffixes match: %s", substring)
        # e.g. "arbeidet" / "skrevet"
        return 0
    if utils.is_grammatical_suffix(substring[nucleus.start() :]):
        logging.debug(
            "the rhyming part is a grammatical suffix: %s", substring[nucleus.start() :]
        )
        # e.g. "blomster" / "fester"
        return 0

    if not sequence1.endswith(substring) or not sequence2.endswith(substring):
        # not an end rhyme
        logging.debug("not an end rhyme: %s and %s", sequence1, sequence2)
        return 0
    if substring == sequence1 or substring == sequence2:
        # one of the words is fully contained in the other
        logging.debug("Nødrim: %s and %s", sequence1, sequence2)
        return 0.5

    if nucleus and (sequence1 != sequence2):
        logging.debug("Proper rhyme: %s and %s", sequence1, sequence2)
        return 1
    # otherwise, assume that the words do not rhyme
    logging.debug("No condition met for a rhyme: %s and %s", sequence1, sequence2)
    return 0


def longest_common_substring(string1: str, string2: str) -> str:
    """Find the longest common substring between two strings.

    Implementation based on the pseudocode from:
    https://en.wikipedia.org/wiki/Longest_common_substring#Dynamic_programming
    """
    m = len(string1)
    n = len(string2)
    L = np.zeros((m + 1, n + 1))
    z = 0
    result = ""

    for i in range(1, m + 1):
        for j in range(1, n + 1):
            if string1[i - 1] == string2[j - 1]:
                L[i][j] = L[i - 1][j - 1] + 1
                if L[i][j] > z:
                    z = L[i][j]
                    result = string1[(i - int(z)) : i]
            else:
                L[i][j] = 0
    return result


def shared_ending_substring(string1: str, string2: str) -> str:
    """Find the shared substring at the end of two strings."""
    min_length = min(len(string1), len(string2))

    for i in range(1, min_length + 1):
        if string1[-i] != string2[-i]:
            return string1[-i + 1 :] if i > 1 else ""
    return string1[-min_length:] if min_length > 0 else ""


def find_last_stressed_syllable(syll):
    """Find the last stressed syllable in a list of syllables."""
    n = len(syll)

    for i in range(1, n + 1):
        if re.search(r"[123]", syll[-i]):
            return syll[-i:]
    return syll[:]


def find_rhyming_line(
    current: Verse, previous_lines: list[str], orthographic: bool = False
) -> tuple:
    """Check if the current line rhymes with any of the previous lines."""

    for idx, previous in reversed(list(enumerate(previous_lines))):
        if previous.last_token is None or current.last_token is None:
            continue
        rhyme_score = score_rhyme(
            previous.last_token, current.last_token, orthographic=orthographic
        )
        if rhyme_score > 0:
            return idx, rhyme_score
    return None, 0


def tag_rhyming_verses(verses: list[list[str]], orthographic: bool = False) -> list:
    """Annotate end rhyme patterns in a poem stanza.

    Args:
        verses: list of verselines with words
        orthographic: if True, the words strings are orthographic,
            otherwise assume phonemic nofabet transcriptions
    Return:
        list of annotated verses with rhyme scores and rhyme tags
    """
    alphabet = iter(string.ascii_letters)

    processed = []  # needs to be a list!
    for idx, verseline in enumerate(verses):
        if not verseline:
            continue

        if orthographic:
            tokens = tokenize(verseline)
            try:
                last = (
                    tokens[-1] if not utils.is_punctuation(tokens[-1]) else tokens[-2]
                )
            except IndexError:
                logging.debug("No tokens found in %s", verseline)
                continue
            current_verse = Verse(
                id_=idx,
                text=verseline,
                tokens=tokens,
                last_token=last.casefold(),
            )
        else:
            current_verse = Verse(
                id_=idx,
                transcription="\t".join(verseline),
                tokens=verseline,
                syllables=utils.convert_to_syllables(verseline, ipa=False),
            )

            last_token = " ".join(find_last_stressed_syllable(current_verse.syllables))
            current_verse.last_token = re.sub(r"[0123]", "", last_token)

        rhyming_idx, rhyme_score = find_rhyming_line(
            current_verse, processed, orthographic=orthographic
        )

        if rhyming_idx is not None and rhyme_score > 0:
            rhyming_verse = processed[rhyming_idx]
            current_verse.rhyme_tag = rhyming_verse.rhyme_tag
            current_verse.rhyme_score = rhyme_score
            current_verse.rhymes_with = rhyming_verse.id_

        else:
            try:
                current_verse.rhyme_tag = next(alphabet)
            except StopIteration:
                logging.info(
                    "Ran out of rhyme tags at %s! Initialising new alphabet.", idx
                )
                alphabet = iter(string.ascii_letters)
                current_verse.rhyme_tag = next(alphabet)

        processed.append(current_verse)
    return processed


def collate_rhyme_scheme(annotated_stanza: list) -> str:
    """Join the rhyme tags rom each tagged verse to form a rhyme scheme."""
    return "".join(verse.rhyme_tag for verse in annotated_stanza)


def get_stanzas_from_transcription(
    transcription: dict, orthographic: bool = False
) -> list:
    """Parse a dict of transcribed verse lines and return a list of stanzas."""
    line_ids = [x for x in transcription.keys() if x.startswith("line_")]
    n_lines = len(line_ids)
    logging.debug("Number of lines in poem: %s", n_lines)
    poem = []
    stanza = []
    for line_n in line_ids:
        verse = transcription.get(line_n)
        if (verse is not None) and (len(verse) > 0):
            words, pron = zip(*verse)
            verseline = list(words if orthographic else pron)
            stanza.append(verseline)
        else:
            if len(stanza) == 0:
                continue
            poem.append(stanza)
            stanza = []
    if len(poem) == 0 and len(stanza) > 0:
        poem.append(stanza)
    return poem


def tag_stanzas(stanzas: list, orthographic: bool = False) -> Generator:
    """Iterate over stanzas and tag verses with a rhyme scheme."""
    for idx, stanza in enumerate(stanzas):
        tagged = tag_rhyming_verses(stanza, orthographic=orthographic)
        rhyme_scheme = collate_rhyme_scheme(tagged)

        yield {
            "stanza_id": idx,
            "rhyme_scheme": rhyme_scheme,
            "verses": [verse.dict for verse in tagged],
        }


def tag_poem_file(poem_file: str, write_to_file: bool = False) -> dict:
    """Annotate rhyming schemes in a poem from a file."""
    # Assume that the stanzas are independent of each other
    # and that the rhyme scheme is unique to each stanza

    filepath = Path(poem_file)

    if not filepath.exists():
        raise FileNotFoundError(f"File {filepath} does not exist.")
    if filepath.suffix == ".json":
        poem_text = json.loads(filepath.read_text())
        poem_id = poem_text.get("text_id")
        orthographic = False
        stanzas = get_stanzas_from_transcription(poem_text, orthographic=orthographic)

    elif filepath.suffix == ".txt":
        poem_id = filepath.stem.split("_")[0]
        poem_text = filepath.read_text()
        stanzas = utils.split_stanzas(poem_text)
        orthographic = True

    logging.debug("Tagging poem: %s", poem_id)

    file_annotations = list(tag_stanzas(stanzas, orthographic=orthographic))

    if write_to_file:
        outputfile = filepath.parent / f"{filepath.stem}_rhyme_scheme.json"
        with outputfile.open("w") as f:
            f.write(json.dumps(file_annotations, ensure_ascii=False, indent=4))

        logging.debug(
            "Saved rhyme scheme annotations for poem %s to \n\t%s", poem_id, outputfile
        )
    return file_annotations


# %%


def main():
    """Main function to run the rhyme detection script."""
    import argparse
    from datetime import datetime

    parser = argparse.ArgumentParser(description="Tag rhyme schemes in a poem.")
    parser.add_argument(
        "jsonfile", type=str, help="Path to a json file with phonemic transcriptions."
    )
    parser.add_argument(
        "-v", "--verbose", action="store_true", help="Set logging level to debug."
    )
    args = parser.parse_args()

    if args.verbose:
        today = datetime.today().date()
        logging_file = f"{__file__.split('.')[0]}_{today}.log"
        logging.basicConfig(level=logging.DEBUG, filename=logging_file, filemode="a")

    tag_poem_file(args.jsonfile)


if __name__ == "__main__":
    import doctest

    doctest.testmod()

    main()



import pytest

from poetry_analysis import rhyme_detection as rd
from poetry_analysis import utils


@pytest.mark.skip("Skipping test for now")
def test_rhyme_scheme_gets_tagged_for_multiple_stanzas(example_poem_landsmaal):
    stanzas = utils.split_stanzas(example_poem_landsmaal)

    tagged_stanzas = list(rd.tag_stanzas(stanzas))

    assert len(tagged_stanzas) == 3

    for stanza in tagged_stanzas:
        assert stanza[0].get("rhyme_scheme") == "abcb"



"""Test data"""

import pytest


@pytest.fixture
def transcribed_poem_lines():
    """2132_Forord_no-nb_digibok_2006082400076"""
    lines = [
        [
            "D AX0 R",
            "V AA1 R",
            "UU2 F R EH3 D",
            "IH0",
            "L AH2 N AX0",
            "P OA3",
            "T OAH1 R V AX0",
            "S T OO1 D",
        ],
        [
            "EE1 N",
            "V AH2 N D R AX0 N AX0",
            "S V EH1 N",
            "M EE1",
            "S II1 N",
            "KJ OE2 P M AH0 N S B OO3 D",
        ],
        [
            "AH0 F",
            "SJ IH2 N NX0 AX0",
            "S M YH3 K AX0 R",
            "L OA1",
            "D IH1 S K AX0 N",
            "F UH2 L",
        ],
        [
            "D AX0 R",
            "V AA1 R",
            "B AE2 L T AX0 R",
            "AH0 F",
            "S IH2 L K AX0",
            "OA1",
            "R IH2 NG AX0 R",
            "AH0 F",
            "G UH2 L",
        ],
    ]
    return lines


@pytest.fixture
def orthographic_poem_lines():
    """2132_Forord_no-nb_digibok_2006082400076"""
    lines = [
        ["Der", "var", "Ufred", "i", "Landet", "Paa", "Torvet", "stod"],
        ["En", "vandrende", "Svend", "med", "sin", "Kjøbmandsbod"],
        ["Af", "skinnende", "Smykker", "laa", "Disken", "fuld"],
        ["Der", "var", "Bælter", "af", "Silke", "og", "Ringer", "af", "Guld"],
    ]
    return lines


@pytest.fixture
def example_poem_landsmaal():
    """Poem ID: 2873_Arne_Garborg_no-nb_digibok_2014073108102"""
    return """Kvass som kniv
i daudkjøt flengjande.
Sanningstyrst
mot ljoset trengjande.


Varm av elsk,
som granskog drøymande.
Frisk som foss
Or berget strøymande.


Nyreist Norig
eldfullt byggjande.
Hovding hæv
um tufti tryggjande!
"""


@pytest.fixture
def example_poem_riksmaal():
    """Poem ID: 766_Kjærligheden_no-nb_digibok_2014110308161"""
    return """Kjærligheden ‒ kjendte du dens glød?
Ren som guld fra Herren selv den flød.
Ædelt, høit og helligt som vor Gud
Er det lovens allerstørste bud.


Kjærligheden ‒ kjendte du dens magt,
Sterkere end døds og helveds pakt?
Mange vande kan ei slukke ud
Kjærligheden, som udgår fra Gud.


Kjærligheden ‒ kjendte du dens værd,
Virksomt, ja langt mer end noget sverd?
Al den ildske, som i verden er,
Overvindes, når den kommer nær.


Kjærligheden ‒ kjendte du dens ånd,
Klippesterke, kjærlighedens bånd?
Større dyd på jord ei nævnes kan,
End den ædle kjærlighedens brand!
"""


@pytest.fixture
def example_poem_danish():
    """Poem ID: 3107_EngelskeSocialister_no-nb_digibok_2015102908059
    Only the 3 first stanzas
    """
    return """Henover Byens Tage glide
de sidste Smil, de hendøende Rester
af Dagen og Solen. I Strømninger stride
vælter sig Flodens mudrede Vande,
og som indbuden Gæst til det smudsige Leje,
fra Havets, fra Nordsøens vaade Veje,
sænker sig Taagen over Byen, over Strømmen,
saa kommer Natten, Døden eller Drømmen!


I Læ for Vinden, i Ly for Taagen,
omkring et Kulbaals ulmende Gløder ‒
rapsede Varer derhenne fra Krogen,
hvor Købmanden losser de drægtige Skuder, ‒
sidder et Selskab. Sod paa Skjorten,
knudrede Arme, en tretten, fjorten
Stykker af dem, der lossede Skuden:
Angelsachsernes Blod ruller under Huden.


De mumler dæmpet og suger paa Piben,
Øllet gaar om i de klinkede Kander,
der er Noget paafærde, man vil ud af Kniben,
man har Noget paa Hjerte, vil Nogen paa Livet;
men skønt Armen dirrer, og Pulsen banker,
mangler man Ord for de mange Tanker;
der er Galskab nok, men System er der ikke.
Da rejser en Mand sig med funklende Blikke.
"""


@pytest.fixture
def poem_with_anaphore():
    """Poem ID: 1924_Jeg_ser_no-nb_digibok_2009010803011"""

    return """Jeg ser.

Jeg ser paa den hvide himmel,
jeg ser paa de graablaa skyer,
jeg ser paa den blodige sol.

Dette er altsaa verden.
Dette er altsaa klodernes hjem.

En regndraabe!

Jeg ser paa de høie huse,
jeg ser paa de tusende vinduer,
jeg ser paa det fjerne kirketaarn.

Dette er altsaa jorden.
Dette er altsaa menneskenes hjem.

De graablaa skyer samler sig. Solen blev borte.

Jeg ser paa de velklædte herrer,
jeg ser paa de smilende damer,
jeg ser paa de ludende heste.

Hvor de graablaa skyer blir tunge.

Jeg ser, jeg ser...
Jeg er vist kommet paa en feil klode!
Her er saa underligt...
"""


@pytest.fixture
def poem_with_alliteration():
    """Poem ID: 2735_Sirius_som_Séer_no-nb_digibok_2009010803031"""
    return """Stjerneklare Septembernat
Sees Sirius,
Sydhimlens smukkeste
Stjerne,
Solens skjønneste Søster,
Svæve saa stille,
Straale saa smukt,
Skue sørgmodigt
Slægternes Strid.
Sine samlede Syner
Sender Stjernen Sirius
Sine store Slægtninge:
Solen, Skorpionen,
Stolte, svømmende Svane,
Sydkorset, Saturn,
Som straalende Stjerneskud.
Sirius ser saameget!
Ser Sagas skyhøie Sæde,
‒ Store Skagastølstind ‒
Som sydfor Snehætten staar.
Saga speidende sidder
Ser ‒ samler ‒ skriver.
Samler, som Snorre Sturleson,
Samler, som Seeren Saxo,
Samler stortænkte Skrifter,
‒ Stengamle, støvede Skrifter
Saavelsom senest skrevne –
Skaldene, som siger Sandheden smukkest,
Sender Saga sit stolteste Smil.
Saalangt, som Samfund stiftes,
Ser Saga  samler  skriver.
Saaledes ser Saga:
Store Stater
Svinge sit skarpe Sværd,
Sønderlemme Smaastater,
Snigmyrde Smaafolk,
Som sturende sidder.
Smaafolk, som saares,
Strider, søger sig skjærme
Saavidt Smaafolk skjønner,
Saalangt Synet strækker.
Store Staters Styggedom
Skabte Socialisternes Stræv.
Sligt ser Saga ‒ sukker ‒ skriver.
Stundimellem samler Saga Slægtregistre,
Som senere Slægter ser.
Sommetider Stormagter skjændes,
Som sletikke Saga ser:
Saaledes saa Sirius
Søndenvinden skamskjelde Snedronningen
Sidstleden syvende September.
Søndenvinden sagde spydigt:
“Snedronningen sidder saa stille,
Spiser Spitsbergens skidne Sne,
Skjønt Solen skinner saa smukt
Sydover solrigs Septemberdag.
Seil sydover, skjønne Snedronning!
Se Spaniens smukke Stæder,
Se Spaniens storladne Sæder,
Som sømmer sig stolte Snedronning.
Spis svulmende Sydfrugter,
Smag Spaniens søde Safter,
Sligt skal styrke Segnende,
Skjærpe Svages slappe Smag.
Saadan Stillesidden
Samt stadig Surmulen
Skader sarte Skjønhed.”
“Seil selv sydover, satiriske,
Spottende, springfyragtige Spasmager,”
Svarer Snedronningen stolt,
Saa slipper Snedronningen sligt
Slidder, Sladder, Sladder!
Spitsbergens skinnende Sale
Samt sølvklare Sneboller
Smager styrkende  sundt.
Søndenvinden skal sagtens
Slikke Sydens Stæv,
Søke stikkende Sol.”
“Stive Snerpe”! sagde Søndenvinden.
Sjofelist!
Slu, snedige Slubbert”!
Svarte Snedronningen sint.
Snip, Snap, Snude,
Saa slutter Snedronningens Sagn.


Sceneriet skifter,
Sirius smiler,
Sender Søster Skorpionen
Synerne som Stjerneskud.
Særligt saadanne Syner:
Storthingsmænd sludrer sværtlænge,
Skraber sig Skillinger sammen,
Sløser Statens Sølvmynt,
Sætter Skatten svært stor.
“Sir Søren” staar saa skjændende,
Ser Statskassens Skatte svinde,
Siger sine Samarbeidere Skoser,
Som stelte Skjødebarnet slig.
Sverdrup senior siger stadigt:
“Se saa skal Skabet staa.”
Sognepræst Sverdrup svinger Svøben,
Snerter stygt sine Sidemænd,
Sætter sig selv saa stor.
“Snilde som Slanger”
Staar Schweigaard, Stang;
Stang slaar Stortrommen,
Schweigaard stemmer sin Strængeleg.
Smukke Samspil
Smelter Sjæle,
Saa Stivsindet svinder,
Som Sol smelter Sne.
Slagfærdig staar Storbladsmand Schøien.
Steen ‒ Stavangers Skolemand –
Siger snart saa  snart saa.
Somme sidder svært selvgode,
Soelberg snakker sletikke.
Storpralende Storbønder sidder
Somoftest søvnige  stumme.
Stemmeretten skrinlagdes skammeligt,
‒ Samtlige Stormænds største Streg.
Saaledes Storthingsmænd saares,
Som sliter saa surt,
Søndag som Søgnedag.
Skjæbnerne skifter  Sirius ser
Storthingsmænd stige,
Sæge sig større Stilling,
Sikre sig Statsraadstaburetten,
Staa snart som Statens
Største Styrere,
Styre Statsskibet saa som saa.
Stjernen Sirius smiler skjælmsk,
Sender sine Stjerneskud,
Saasnart svenskefjæskende Statsraader
Steller, stuller, styrer snodigt:
Se saaledes salig Stangs Styretid,
Senere Selmer, Schweigaard,
Stang, Sverdruperne, samt
Saakaldet stumme Stang,
Jom styrtedes sidst.
Skulde saamange store Stænger
‒ Store Slægters store Sønner –
Staa svaiende?
Svigte sit Statskald?
Saagodtsom sælge Staten?
Smigre Svenskerne?
Staa som Statsforrædere?
Sladder! Stort Sluddær!
“Sletingen støtter som store Stænger.”
Siger Storbladene smukt som sandt.
“Skidt san! Slige slingrende Støtter!”
Svarer Smudsbladene spidsfindig.
Steen!
Store Steen skal sandelig staa,
Som Statens solideste Støtte!”
Smudsbladene skreg selvfølgelig:
“Sten! Sten! Sten!”
Sædt smilede Skolemanden Steen,
Sendte Stavangerne Slængkys,
Beilede, skulde snart Storbyen se,
Skulde snart Slægten syne
Stordaad, som sjelden spurgtes.
Snart som Storbyens Stormand
Stod samme Skolemand,
Samlede sit Skrækministerium,
Som skal sætte Svensken Stenen,
‒ Sletikke Stangen –
Sletikke staa som svaiende Stang.
Skrækslagne saa Storbladene
Steen samle slette Subjekter,
Som skal Staten styre.
‒ Slette Subjekters store Synder
Samled Storbladene samvittighedsfuldt,
Saa Slægten skulde se
Statsministerens store Skjævsyn,
Som satte slige sjofle Stakkarer
Som Statsraader.
“Skurkagtigt Svinepak!”
“Sørgeligt Satansværk!”
Sukked Storbladene saart.
Sexsogsyvti slige Skjældsord
Skjæmte stadigt Spalterne.
Slige stygge Skriverier
Sees stadig som
Storpartiets største Styrke.
Samtidig skildred Smaabladene
Statsministeriet Stang.
Strax sagde Storbladene:
“Smudsbladene skumler,
Skriver stygt, samvittighedsløst.
Siger saamange Storløgne,
Som skal skræmme svage Sjæle.
Smudspressen sparker saa skrækkeligt
Storbladenes Stormænd,
Søndertrampes, sønderknuses, sønderrives
Skulde sligt Skadedyr,
Søndenvinden Stumperne sprede,
Saa Styggeriet svandt
Som Søens salte Skum.”
Saadanne stygge Stiklerier
Storbladene stadig skriver,
Siger Smaabladene saameget skarpt,
Som skal Smaafolk skræmme.
Smaabladene, stakkar,
Svarer sjelden,
Skriver skaansomt,
Saarer sletikke.
Sandelig stor Synd,
Smaabladene skal
Skamskjændes slig!


Som sagt,
Statsskibet styres saa som saa:
Somme ser Stormen
Skuden slænge,
Søuhyrer svære
Skuden sluge,
Søen sprøite, skumme,
Saa Skuden synkefærdig synes.
Sørgelige Spaadomme
Spørges sommetider.
Saaledes siges som sikkert:
Storrussen skal snart
Sværdet svinge,
Sønderlemme,
Sønderhugge
Statsskibet, som styres saa slet.
Splitte Slægter,
Snigmyrde, stjæle
Stort som Smaat;
Sende Størsteparten Søveien,
Som Storruslands selvtagne Skat.
Spydige Slængord
Slaar stakkars Statsraader
Som Stenkast.
Skamfuld staar Statsraadflokken.
Selvforskyldte Svøbeslag
Smager sletikke.
Somme synes sandelig
Skuden styres smukt,
Synes Statsraader skalter,
Slig som sig sømmer
Selvstændigt Styre.
(“Se Skandalpressen”
Siger Storbladene.)
Sandsynligvis sker snart som spaaet:
Statsraader skynder sig
Sin Statsraadtitel sælge,
Sæge sin simple Stand,
Sin svundne Stilling;
Staar snart som Sognepræster,
Snart som Skolemænd,
Snart som Skrivere,
Somme som Storbønder
Skjuler sig ‒ svinder
Som sjælløse Skygger.
Skulde synes,
Slige stakkars svundne Storheder
Skulde skræmme,
Saa Slægten senerehen
Stadigt saa sit stolteste Syn, –
Saa Storbladsmænd
Som Statens Styrere!
Storhjertede Storbladsmænd,
Som sidder stivryggede,
Smukke, staute, solide,
Slig som sig sømmer
Stockholmsfarere, samt
Slottets som Stiftsgaardens Størrelser.
“Saadant ske snarest!”
Sukker Storbladene.
Statsraadhuden synes sandelig stenhaard!
Sirius ser Striden!


Sognepræsterne synes
Sognebørn sparer saameget:
“Sænder saa sjelden Smør,
Bauer, samt større Steger.
Sjælesørgere skal skattes,
Sendes Stymperes sidste
Styver.”
Saaledes snakker Sorte
kjolerne
Sagtelig sigimellem.
Sognefolk skulde sig
skamme !
Sværte sine Sjælehyrder slig!
Sirius ser stadigt skiftende Syner:
Ser Skuespillere sværtmange Steder
Spille skrækkelig slet,
Saa skuelystne Skarer
Sletintet skjønner.
Saaledes spiller sletikke
Schrøders Skuespillere,
Som studerer sine Sager;
Spiller selvfølgelig smukt
Saamangen Storaands Syner,
Saamange storslagne Sørgespil.
Samtidig sløifer
Schrøders Scene
Skurrile stykker.
Saaledes siger Schrøder:
“Saavel Storbjørnen som
Skribenten, som skrev
“Samfundets Støtter”,
Skal sletikke se
Sine samlede Skuespil;
Sligt Snaus
Skal sandelig spilles sjelden.
Scenen skal samle
Smaa skjælmske Skovaber,
Som skaber sig!
Sligt styrker Skuespilsansen,
Styrker Samspillet,
Skaffer saamangen sund Skoggerlatter.”


Storbjørnen skjæmmer sig storligt,
Stakkar!
Skifter saaofte Sind:
Staar snart som Slingringsmand,
Slænger snart som Stormhanen,
Staar stundom stiv, stram, strunk,
Skravler stundom som Spaakjærring,
Skulde stille, sympathetisk, standhaftig
Som Skjeidkampen staa,
Saa skulde Skalden stille
Slægtens store Sjælevunde,
Stills saamange skjulte Savn.
Snart synger Skalden
Smukke Sange,
Snart sender Skalden
Særsynte Skrifter,
Som sætter Skræk,
Skildrer Samfundssynderne stærkt,
Straffer strængt.
Snart som Skribent,
Snart som Stortaler
Skildrer Storbjørnen selvgod sin Samtid.
Søndagsfreden skildres smukt,
Søgnedagsstriden skildres sandt,
Storpolitisk Sludder skrives somoftest, ‒
Svaier saaledes selv som Siv.
Smaastæders Skjødesynd, Sladder,
Skaaner sletingen Skjønaand.
Sandheden skal siges:
Skjønneste Syn Sirius saa –
Slægtskjærligheden
Sætter Storbjørnen størst:
Steller store Søstre smukt
Sine smaa svage Søskende,
Sender Sirius strax
Særegne Stjerneskud,
Som skildrer Sydkorset
Slige Søskendes Samliv.
Saafremt store Sønner
Skaffer sin Slægt Sorg,
Skjuler Sirius sig strax.
Som stor, sort Stenmur
Stænger svære Skymasser
Samtlige Stjerners Skjønhed,
Saa slige slemme Sønner
Staar som stærblinde,
Ser ‒ sort – sort ‒ sort.
Strængt straffer Skjæbnens Styrer
Slægtens skammeligste Synd,
Sender saaledes Sygeleie,
Sværtmange slemme Sygdomme,
Store Smerter, Sult,
Sorger, saamange Slags Savn,
Som slider, sønderriver,
Skjærer slemme Saar,
Som svulmer, svider.
Sommetider sendes Snedrev,
Storm, Skypumper, Stormflod,
Som seigpiner samtlige,
Saa sidste Søvn synes sød.
Sligt sees stadigt,
Skjønt Sekler svandt
Siden sidste store Syndflod.
Syndfloden, som skyllede,
Sopte ‒ steg ‒ steg ‒ steg –
Saaat svaiende smaa Sivplanter
Saavelsom skyhøie Snefjelde skjultes.
Største Sømand, Sirius saa,
Sparedes, samt Sømandens
Skyldfri Sønner.
Sømanden ‒ senere Slægters
Stamfader ‒ sparedes,
Samt Sømandens skyldfri Slægt.
Sirius svæver saa stille,
Straaler saa smukt,
Skuer sørgmodigt
Slægternes Strid,
Skjuler sig somoftest,
Saasnart Skjæbnens Styrer
Sender slige svære Straffedomme.
Storbjørnen ser som Sirius!
Som Sirius sender sine Stjerneskud,
Saaledes skildrer Storbjørnen
Samtiden sine Syner.
Syndens sanseberusede Slavinder
Ser saare sjelden
Solen skinne,
Stjerner straale ‒ smile.
Saadanne syndige Stakler
Staar stærblinde,
Ser som Sønnerne
Sort ‒ Sort ‒ Sort.
Standhaftighed!
Slutter Spindesiden sig sammen,
Skal snart Skandalvæsenet svinde,
Saa Staklerne staar seende,
Som skikkelige Samfundslemmer.


Skjønhedssansen staar svært stille:
Saaledes smukkeste Statue
Skildres som stygt Skabilken,
Stemningsfulde Studier slænges
Som styggeste Skilderi.
Skjønliteraturen staar saa som saa:
Somme samler Skandalhistorier,
Sætter sammen – skriver ‒
Sælger sligt som Skjønliteratur.
Skeptikeren skriver sine Synsmaader,
Sofisten sine Slutninger,
Stoikeren sine strænge Sædelighedskrav,
‒ Samtlige synes, sit Smøreri
Staar som sandeste Skjønliteratur.
Saare sjelden sees Skjønliteraturen
Samle Stoffet smukt,
Skjærpe Synet,
Skjænke Sjælen Skjønhedsindtryk,
Som sent svinder.
Studerte Smaaskribenterne
Sappho,
S-Zola,
Shakespeare,
Schiller,
Schandorph,
Strindberg,
Sivle,
Saaes sandelig sjeldnere
Saamange slette Skrifter.
Sidstnævnte syv Skalde
Sendte  som sender ‒
Skrifter, som sent skal
Smuldres som Støv.
Saadanne Skrifter,
Som samtlige sendte,
Skildrer Sjælelivet smukt,
Skildrer Sorgerne sandt,
Skildrer Striden selvstændigt,
Skildrer Seieren straalende,
Skilsmissesager sympathetisk,
Situationer saftigt;
Stemningsfuldt skildres
Salige Stunder.
Stilen storslagen skjøn.


Satirens Svøbe
Skulde sletikke skaane
Skrivelystne Stymperel!
Saadant ser Sirius.
Sommetider ser Sirius
Stakkels smaa Studenter,
Som sidder sultne,
Studerer sig syge.
Slige stakkels Studenter
Studerer særligt Stjernevidenskaben,
Ser Solnedgangen,
Sidder studsende,
Spørger sig selv saaledes:
Sodiakallyset?
So ‒ di ‒ a ‒ kal ‒ ly ‒ set ‒?
Skjønner sletikke sligt!
Som stille Snefald
Sees Sodiakallyset,
Saasnart Solen sig skjuler.
“Synes”, siger Studenten,
“Synes, Sodiakallyset
Staar som Stjernen Saturns
Skjønne Straalering?
Sikkert selvsamme Stof!”
Siger Studenten sjæleglad.
Studenten staar stille,
Ser snart Saturn,
Snart Sodiakallyset,
Sammenligner,
Smiler selvtilfreds.
Sluttelig søger Studenten
Stivfrossen sin Seng,
Sover snart sødelig.
Siriusstraalerne skiftede smukt.
Studentens søde Smaasnak
Sendte Sirius strax
Sin stolte Søster Svanen.
Svanen snaddrede selsomt,
Smaalo, svømmede sydligt:
Søde Søvn svinder snart,
Stenhaarde Søvn sig sænker,
Skaffer Studenten slemme Syner.
Stakkels Student spreller,
Snorker skrækkeligt,
Snakker, slaar, spender
Sin Sengkammerat,
Skriger stundimellem saa:
“Spæktralanalyse!
Spektroskop!
Se Solspektret, Schiaparelli!
Solpletter, Solfakler, Sol, Sol!”
“Stop!” skraaler Sengkammeraten,
“Slig sindssvag Snak
Skræmmer Søster Stine,
Som sidder selvanden,
Syr Sømandsdragt samt
Særker sex,
Som skal sendes Statsraadinden,
Saasnart Stjernehæren svinder,
Solen sine Straaler sender.
Skræmmer saadan Støi
Stakkels Søster Stine,
Syes Svømmedragten sent,
Som skal sendes senest sex.
Saasnart Slaguhret slaar syv,
Seiler Statsraadinden,
Beiler sydtil Sandefjord,
Søgende sin Sundhed;
Søbadet skal skaffe
Sundheden, som svandt.
Skjønne Statsraadinde!
Smukke, slanke Statsraadinde
Skal sletikke staa
Som Susanna splitternøgen!”
Sovende svarer Studenten:
“Sludder!
Splitternøgen Statsraadinde!
Svømmedragt, Særk!
Sludder!
Solfakkel, Sol, Sol!”


Stadig ser Sirius skiftende
Syner:
Ser saameget snurrigt
‒ Skjønt som stygt –
Ser Sladresøstre
Støiende sidde sammen,
Skravle, skrige, skryte,
skraale,
Sværte stygt sine
Søskende.
Samtidig strikker Søstrene,
Som sidder saaledes
sammen,
Søde smaabitte Strømper,
Som sorte Sulubørn slider.
Samtidig syr Søstrene,
Som sladrer saaledes sammen,
Store, svære Skjorter,
Som stormvante Sømænd slider.
Somme sætter Sulumissionen,
Somme sætter Sømandsmissionen,
Somme sætter Santalmissionen
Som Slægtens største Sag.
Sirius ser sligt, som sjelden sker,
Saavelsom Smaatterier,
Ser Smaatterier
Saavelsom Storstadsherligheder.
Ser saaledes Sminken
Skjule store Skrøbeligheder,
Baa selv stygge Skygger
Sees som Skjønheder.
Ser Snørlivet, som skaber
Slanke Sylfider,
Smækkre Sylfider,
Som slangeagtigt snor sin
Smukke Svanehals.
Sirius ser saameget:
Ber Slagterne slaa
Studene skrækkelig stygt,
Synge slibrige Sange,
Som skræmmer Smaajenterne slemt.
Skræddersvendene sender
Sin stygge, sjuskede Søm,
Skomagerdrengene stjæler
Svendenes Saalelæder.
Smed slog Satansnødden
Som Sopelimspinden stængte,
Saa Smietaget strøg;
Storslæggas Smaastumper
Saaes senere spredte
Søndenfor Skudesnæs.
Stupiger, Sypiger, Sergeanter
Slaaes sluttelig sammen,
Seiler saa sin Sø.
“Særlaget” samler saadanne smukt.
Skyhøit sætter somme ‒
“Særlagets” Stifter,
Somme snakker stygt  syndigt,
Siger:
“Særlagets” Stifter søger Strid,
Sætter Splid,
Snakker saameget Sludder,
Sløver selvfølgelig
Svagsinte Smaapigers
Sunde Sands.
Stifteren skulde sandfærdig sendes
Saalangt, som Sibiriens Sneørkner sig
Straffende strækker.”


Septembermaanederne
Skaffer somoftest
Sirius snurrige Syner:
Saaledes ser Sirius
Skolefolk sidde sammen,
Studere, stirre, sige som saa:
“Straffes skal Syndere smaa,
Saa slipper Samfundet senere
Saamange store Syndere,
Som sælger sin Sjæl,
Soner sin svære Straf.”
“Spot som Skam slig Straffelyst!”
Svarer Sagfører Sørensens,
“Smaabørn skal sletikke straffes,
Sletikke slaaes!”
Sagfører Sørensens samlede, skrev
Skrækkelige Sladrehistorier sammen,
Som skulde Skolefolk syne
Sin store Skyld,
Som slog slig søde smaa Stakler,
Saa Spanskrøret sønder sprang.
Somme smaa Stakler sloges syge,
Somme smaa Stakler spottedes saart.
“Sligt Skolevæsen skal stoppes,”
Skrev Sørensens stadig strængt.
“Særensens Skriverier skal stoppes!”
Skreg samlede Skolelærerstand,
“Sagsøges skal sandelig
Sagførerparret!”
Som sagt saa skede.
(Se Sørensens Sag,
Som sluttede sexogtyvende September.)
Somme syntes Sørensens Sag stod slet,
Syntes Sørensens skulde
Satisficere
Samtlige Skolelærere.
Skolelærerne saa
Stor Sympathi,
Skjønt Sørensens Sag seirede;
Somme sagde saagar:
“Schjødt stelte sandelig Sagen slig.
Som Schjødt selv syntes,
Skjældte, smældte,
Spurgte spidsfindigt,
Sendte snartænkt
Spydige Snerter,
Saa Staklerne stode
Sindsforvirrede,
Skamfulde, stumme.
Selvfølgelig staar Sørensens
Skyldfri, straffri,
Seiersstolte.”
Saaledes skrev Storbladene.
Smaabladene studerte stadigt
Sagens sande Sammenhæng,
Samlede store Sandheder,
‒ Saa Skyggesiderne ‒
Summerte sluttelig saa:
“Stor Sag –
Stor Sensation!
Sørensens stred storhjertet
Simple Smaafolks Sag.
Sagkyndig seet
Skulde Sørensens seire.”
Schjødt smilte sagtmodig,
Sagde stilfærdig:
“Styg, smaalig Snak
Skader sletikke
Stærke Skuldre.”
Se, slig Strid ser Sirius.



Sommetider ser Sirius
Saameget smaat, snevert,
Ser saaledes syv smale
Spalter,
Som sluger saamegen
Smaamynt.
Snille Schibsted! Sæt
Spalter sex!
Snille Schibsted! Sløif
syvende Spalte!
Sæt Spalterne slig,
Som Spalterne stod
Sidste September.
Smilende ser Sirius
Somoftest saameget smukt:
Ser Solstraaler spille,
Ser Snefaanner svinde,
Ser Smaablomster spire,
Skyde, sætte Stængel,
Ser Sommerfugle slikke
Sol, som Stængelens Stæv,
Ser Skovalfer sværve,
Skovsangerne smaakviddre,
Søerne sødelig smile,
Skov som Skrænter speile,
Stormen Skyerne sprede,
Solstreif skyndsomst stille
Skabningens skjulte Suk.
Støvøiet synes, sligt staar smukt,
Siriusøiet samler saadant smukkere.
Sommetider ser Sirius
Selskaber slutte sig sammen,
Spadsere store Strækninger ‒
Søgende sjeldne smukke Steder,
Som skal sees, studeres.
Stien slynger sig
Stærkt stigende.
Steilt Skrænterne sænker sig.
Saasnart Stenheller sees,
Søger samtlige Siddeplads.
Spiselige Sager serveres,
Snapser supes,
‒ Somme super Sherry ‒
Somme spiser,
Somme spaser,
Somme skjændes,
Somme slaaes.
Støien skræmte
Smaafuglene, stakkar,
Som sad stille syngende,
Slog sine søde Smaatriller,
Som stilned smaaningom.
Somme sidder stille,
Skriver Stedet smukke Sange;
Somme stiller Staffeliet,
Smører Studier sammen.
Sortkridtet skitserer
Snefjeldet som Søen,
Snebræen som Skoven,
Snevandet som samler sig,
Styrter, suser,
Slynger sig,
Skaber store Stryg,
Som skyndsomst søger
Sørfjorden, smilende, speilblank.
Se saaledes:
Stendalsfossen.
Sjelden smuk stod Solopgangen,
Solnedgangen smukkere.
Sidste Straale
Sendte Stedet
Slumrekys.
Septemberkvælden
Sænkede sit sorte Slør.
Selskabet sig skynder
Søge Stien sydover.
Somme snakked,
Somme sang,
Somme sagde:
“Se Stjernerne!”
“Se Sirius!
‒ Storhunden ‒
Straalende smuk,
Svæver saa stille,
Ser Slægternes Strid.”
Stendalsfossens svindende Sus
Samlede Sindene,
Sendte som Sirius
Stemningsfuld Stilhed.
Siriussynerne skifter stadigt:
Saaledes ser Sirius
Sygdom slide
Syndere, som sukke,
Ser Sygesøstre sidde stille,
Synge smukke Salmer,
Ser Synderes Sjæle
Salige svæve,
Skinne som Sole,
Straale som Stjerner.
Sandheden seirende staar,
Synd som Sorger svundet,
Striden sødelig stilnet,
Skilte sammenføiet.
Smilende, skiftende Sted
Ser Sirius saameget smukt,
Sender Sydkorset Synerne
Som straalende Stjerneskud.

Slut.
"""



import pytest

from poetry_analysis import rhyme_detection as rd


@pytest.fixture
def previous_transcribed_lines(transcribed_poem_lines):
    """Create a list of Verse objects from the transcribed lines"""
    last_tokens = ["S T OO1 D", "B OO3 D", "F UH2 L", "G UH2 L"]
    verselines = [
        rd.Verse(idx, tokens=words, last_token=last)
        for idx, (words, last) in enumerate(zip(transcribed_poem_lines, last_tokens))
    ]
    return verselines


@pytest.fixture
def previous_orthographic_lines(orthographic_poem_lines):
    verselines = [
        rd.Verse(idx, tokens=words, last_token=words[-1])
        for idx, words in enumerate(orthographic_poem_lines)
    ]
    return verselines


def test_find_rhyming_transcribed_line_returns_None_and_zero_if_not_found(
    previous_transcribed_lines,
):
    # Given
    current_line = rd.Verse("X", last_token="X", tokens=["X"])

    # When
    verse, score = rd.find_rhyming_line(current_line, previous_transcribed_lines)

    # Then
    assert verse is None
    assert score == 0


def test_find_rhyming_transcribed_line_returns_correct_rhyming_line(
    previous_transcribed_lines,
):
    # Given
    current_line = rd.Verse("X", last_token="J UH2 L")

    # When
    verse, score = rd.find_rhyming_line(current_line, previous_transcribed_lines)

    # Then
    # It will return the first rhyming match it finds, i.e. the last line
    assert verse == 3
    assert score == 1
    assert previous_transcribed_lines[verse].last_token == "G UH2 L"


def test_finds_rhyming_orthographic_line(
    previous_orthographic_lines,
):
    # Given
    current_line = rd.Verse("X", last_token="tuld")

    # When
    verse, score = rd.find_rhyming_line(
        current_line, previous_orthographic_lines, orthographic=True
    )

    # Then
    # It will return the first rhyming match it finds, i.e. the last line
    assert verse == 3
    assert score == 1
    assert previous_orthographic_lines[verse].last_token == "Guld"



import pytest

from poetry_analysis import rhyme_detection as rd


def test_get_stanza_from_transcription_returns_only_orthogtraphic_words():
    # Given
    poem = {
        "text_id": "test_1",
        "line_0": [["E", "E2"], ["m", "M"]],
        "line_1": [["Det", "D AX0"]],
    }
    # When
    result = rd.get_stanzas_from_transcription(poem, orthographic=True)
    # Then
    assert result[0][0] == ["E", "m"]
    assert result[0][1] == ["Det"]


def test_get_stanza_from_transcription_returns_only_pronunciation():
    # Given
    poem = {
        "text_id": "test_2",
        "line_0": [["Ensom", "EE2 N S AH0 M"]],
        "line_1": [["Det", "D AX0"]],
    }
    # When
    result = rd.get_stanzas_from_transcription(poem, orthographic=False)
    # Then
    assert result[0][0] == ["EE2 N S AH0 M"]
    assert result[0][1] == ["D AX0"]


@pytest.fixture
def transcription_output():
    return {
        "X": "bla",
        "Y": "di",
        "z": "da",
        "line_0": [(1, "line")],
        "line_1": [(2, "line")],
    }


def test_extracting_only_first_elements_in_tuples(transcription_output):
    stanzas = rd.get_stanzas_from_transcription(transcription_output, orthographic=True)
    verses = stanzas[0]
    verse1 = verses[0][0]
    verse2 = verses[1][0]
    assert verse1 == 1
    assert verse2 == 2


def test_extracting_second(transcription_output):
    stanzas = rd.get_stanzas_from_transcription(
        transcription_output, orthographic=False
    )
    verses = stanzas[0]
    assert all(obj == "line" for line in verses for obj in line)



from collections import Counter

from poetry_analysis.anaphora import count_initial_phrases


def test_count_initial_phrases_returns_increasing_phrases_with_single_counts():
    text = "ett ord blir flere til en hel linje"
    expected = Counter(
        {
            "ett": 1,
            "ett ord": 1,
            "ett ord blir": 1,
            "ett ord blir flere": 1,
            "ett ord blir flere til": 1,
            "ett ord blir flere til en": 1,
            "ett ord blir flere til en hel": 1,
            "ett ord blir flere til en hel linje": 1,
        }
    )
    result = count_initial_phrases(text)
    assert all(count == 1 for count in result.values())
    assert all(
        actual_phrase == expected_phrase
        for actual_phrase, expected_phrase in zip(result.keys(), expected.keys())
    )


def test_repeated_phrases_are_counted():
    text = "ett ord i en frase, ett ord i en linje, ett ord i en setning"
    result = count_initial_phrases(text)

    assert result["ett"] == 3
    assert result["ett ord"] == 3
    assert result["ett ord i"] == 3
    assert result["ett ord i en"] == 3
    assert result["ett ord i en frase"] == 1
    assert result.most_common(1) == [("ett", 3)]



from poetry_analysis.alliteration import count_alliteration


def test_alliteration_returns_initial_consonant_counts():
    """Test that the function returns the count of word-initial consonants."""
    # Given
    text = """Stjerneklare Septembernat
Sees Sirius, 
Sydhimlens smukkeste 
Stjerne, 
Solens skjønneste Søster, 
Svæve saa stille, 
Straale saa smukt, 
Skue sørgmodigt 
Slægternes Strid.
"""
    expected = {"s": 20}
    # When
    result = count_alliteration(text)
    # Then
    assert result == expected


def test_count_alliteration_counts_only_words_with_alliteration():
    # Given
    text = "Her er noen ord som ikke gir alliterasjon"
    # When
    result = count_alliteration(text)
    # Then
    assert result == {}



import pytest

from poetry_analysis import rhyme_detection as rd


def test_shared_ending_substring():
    string1 = "skamfuld"
    string2 = "skamguld"

    result = rd.shared_ending_substring(string1, string2)

    assert result == "uld"


def test_shared_ending_substring_no_match():
    string1 = "skamfullt"
    string2 = "skammen"

    result = rd.shared_ending_substring(string1, string2)

    assert result == ""


def test_shared_ending_substring_transcription():
    string1 = "S T OO D"
    string2 = "B OO D"

    result = rd.shared_ending_substring(string1, string2).strip()

    assert result == "OO D"


@pytest.mark.parametrize(
    "sequence1, sequence2",
    [
        ("klangen", "klang"),
        ("klang", "klangen"),
        ("arbeider", "arbeidene"),
        ("klangen", "sang"),
        ("sang", "klangen"),
    ],
)
def test_no_shared_ending_substring_is_found(sequence1, sequence2):
    result = rd.shared_ending_substring(sequence1, sequence2)
    assert not result


@pytest.mark.parametrize(
    "sequence1, sequence2",
    [
        (["F", "R", "YY1", "D"], ["S", "YY1", "D"]),
        (["S", "II1", "N"], ["D", "II2", "N"]),
        ("F R YY D", "S YY D"),
        ("S II N", "D II N"),
        ("G UH L", "J UH L"),
        ("G UH2 L", "F UH2 L"),
        ("B OO D", "S T OO D"),
    ],
)
def test_shared_ending_substring_phonemic(sequence1, sequence2):
    result = rd.shared_ending_substring(sequence1, sequence2)
    assert result



import pytest

from poetry_analysis import rhyme_detection as rt


@pytest.mark.parametrize(
    "syllable1, syllable2",
    [
        [["EE1", "N"], ["EE1", "N"]],
        [["T", "UH1"], ["T", "UH1"]],
    ],
)
def test_returns_True_when_syllable_sequences_are_identical(syllable1, syllable2):
    result = rt.do_syll_seqs_rhyme(syllable1, syllable2)
    assert result


@pytest.mark.parametrize(
    "syllable1, syllable2",
    [
        [
            ["S", "T", "A1", "V", "E2", "L", "S", "E0"],
            ["S", "T", "A2", "V", "E3", "L", "S", "E1"],
        ],
        [["L", "IH3", "K"], ["L", "IH2", "K"]],
        [["AX0", "N"], ["AX1", "N"]],
    ],
)
def test_returns_True_when_same_vowels_have_different_stress_marker(
    syllable1, syllable2
):
    result = rt.do_syll_seqs_rhyme(syllable1, syllable2)
    assert result


@pytest.mark.parametrize(
    "syllable1, syllable2",
    [
        [["S", "T", "A1", "V", "E2", "L", "S", "E0"], ["U", "L", "I1", "K"]],
        [["N", "EH1", "S", "T", "AX0", "N"], ["N", "EE1", "S", "T", "EE0", "N"]],
    ],
)
def test_returns_false_when_syllable_sequences_are_different(syllable1, syllable2):
    result = rt.do_syll_seqs_rhyme(syllable1, syllable2)
    assert not result



import pytest

from poetry_analysis.anaphora import extract_anaphora


@pytest.mark.skip("Deprecated")
@pytest.mark.parametrize(
    "text",
    (
        "jeg ser på den hvite himmel,\njeg ser på de gråblå skyer,\njeg ser på denne blodige sol.",
        "dette er altså verden.\ndette er altså klodernes hjem.\nDette er altså et kjent dikt.",
        "Her er vi i Norge.\nHer er vi på berget.\nHer er vi.\n",
    ),
)
def test_extract_anaphora_returns_3gram_frequencies_of_3_repeating_word_seqs(text):
    result = extract_anaphora(text)
    print(result)
    assert result.keys() == pytest.approx(["1-grams", "2-grams", "3-grams"])
    assert len(result.values()) == 3
    assert result["3-grams"].values() == pytest.approx([3])


@pytest.mark.skip("Deprecated")
def test_extract_anaphora_returns_nothing_if_no_words_repeat():
    text = "Her er noen ord som ikke gir anafora\nFor det er ikke noe som gjentar seg\ni denne teksten."
    result = extract_anaphora(text)
    assert result == {}



"""Test the transcriber module."""

import pytest

from poetry_analysis import utils


@pytest.mark.parametrize(
    "indata, expected",
    [(",", True), (".", True), ("a", False), ("-", True), ("!", True), ("–", True)],
)
def test_is_punctuation(indata, expected):
    """Test that punctuation is correctly identified."""
    # when
    result = utils.is_punctuation(indata)
    # then
    assert result == expected


def test_strip_punctuation():
    """Test that punctuation and additional white space is stripped from a text."""
    text = "De; mumler, dæmpet – og: suger paa Piben,"
    expected = "De mumler dæmpet og suger paa Piben"
    result = utils.strip_punctuation(text)
    assert result == expected


@pytest.mark.parametrize(
    "indata, expected",
    [
        ([["Kvass", "K V AH2 S"], ["som", "S OAH0 M"], ["kniv", "K N II1 V"]], 3),
        (
            [
                ["i", "IH0"],
                ["daudkjøt", "D AEW2 KJ OE3 T"],
                ["flengjande", "F L EH2 N J AH0 N D AX0"],
            ],
            6,
        ),
        ([["Sanningstyrst", "S AH2 N IH0 NG S T YY0 RS RT"]], 3),
        (
            [
                ["mot", "M OO1 T"],
                ["ljoset", "J OO1 S AX0"],
                ["trengjande", "T R EH2 N J AH0 N D AX0"],
            ],
            6,
        ),
    ],
)
def test_syllabify_returns_list_of_syllables(indata, expected):
    """Test that text gets split into syllables"""
    # when
    result = utils.syllabify(indata)
    # then
    assert len(result) == expected


@pytest.mark.skip("Not fully implemented yet")
def test_split_orthographic_text_into_syllables():
    """Test that orthographic text gets split into syllables"""
    # given
    words = ["ensom", "journalist", "ouverture"]
    syllables = [["en", "som"], ["jour", "na", "list"], ["ou", "ver", "tu", "re"]]
    # when
    result = utils.split_orthographic_text_into_syllables(words)
    # then
    for actual, syllables in zip(result, syllables):
        assert actual == syllables



from collections import Counter

from poetry_analysis.anaphora import find_longest_most_frequent_anaphora


def test_find_most_repeating_sequence():
    # Given
    phrases = Counter({"hello": 3, "hello world": 2, "world": 1})
    expected_phrase = "hello"
    expected_count = 3

    # When
    actual_phrase, actual_count = find_longest_most_frequent_anaphora(phrases)

    # Then
    assert actual_phrase == expected_phrase
    assert actual_count == expected_count


def test_find_longest_repeating_sequence():
    # Given
    phrases = Counter({"hello": 2, "hello world": 2, "world": 1})
    expected_phrase = "hello world"
    expected_count = 2

    # When
    actual_phrase, actual_count = find_longest_most_frequent_anaphora(phrases)

    # Then
    assert actual_phrase == expected_phrase
    assert actual_count == expected_count


def test_find_longest_most_repeating_sequence():
    # Given
    phrases = Counter({"hello": 3, "hello world": 3, "world": 1})
    expected_phrase = "hello world"
    expected_count = 3

    # When
    actual_phrase, actual_count = find_longest_most_frequent_anaphora(phrases)

    # Then
    assert actual_phrase == expected_phrase
    assert actual_count == expected_count


def test_ignores_longer_sequence_with_lower_count():
    # Given
    phrases = Counter({"hello": 3, "hello world": 3, "hello world hello world": 1})
    expected_phrase = "hello world"
    expected_count = 3

    # When
    actual_phrase, actual_count = find_longest_most_frequent_anaphora(phrases)

    # Then
    assert actual_phrase == expected_phrase
    assert actual_count == expected_count


def test_find_longest_repeating_sequence_returns_None_with_empty_counter():
    # Given
    phrases = Counter()

    # When
    actual_phrase, actual_count = find_longest_most_frequent_anaphora(phrases)
    # Then
    assert actual_phrase is None
    assert actual_count == 0


def test_find_longest_most_frequent_anaphora_returns_most_frequent_count_instead_of_longest():
    # Given
    phrases = Counter({"hello": 5, "hello world": 3, "world": 2})
    expected_phrase = "hello"
    expected_count = 5

    # When
    actual_phrase, actual_count = find_longest_most_frequent_anaphora(phrases)

    # Then
    assert actual_phrase == expected_phrase
    assert actual_count == expected_count



import pytest

from poetry_analysis.utils import split_stanzas


@pytest.mark.parametrize(
    "example_fixture, n_stanzas, n_lines",
    [
        ("example_poem_danish", 3, 8),
        ("example_poem_riksmaal", 4, 4),
        ("example_poem_landsmaal", 3, 4),
    ],
)
def test_multi_stanza_poem_is_split_into_correct_number_of_stanzas(
    example_fixture, n_stanzas, n_lines, request
):
    # given example poems
    example = request.getfixturevalue(example_fixture)
    # when
    result = split_stanzas(example)
    # then
    assert len(result) == n_stanzas  # Number of stanzas in the poem
    assert all(
        len(stanza) == n_lines for stanza in result
    )  # Number of lines in each stanza



import pytest

from poetry_analysis.alliteration import extract_alliteration


def test_alliteration_returns_initial_consonant_counts():
    # Given
    text = [
        line.strip()
        for line in """Stjerneklare Septembernat
    Skaldene som siger sandhetden smukkest
    Ser Samla saga skriver.
    Litt flere ord
    """.splitlines()
    ]

    expected = [
        {
            "line": 0,
            "symbol": "s",
            "count": 2,
            "words": ["Stjerneklare", "Septembernat"],
        },
        {
            "line": 1,
            "symbol": "s",
            "count": 5,
            "words": ["Skaldene", "som", "siger", "sandhetden", "smukkest"],
        },
        {
            "line": 2,
            "symbol": "s",
            "count": 4,
            "words": ["Ser", "Samla", "saga", "skriver."],
        },
    ]

    # When
    result = extract_alliteration(text)
    assert pytest.approx(expected) == result


def test_extract_alliteration_skips_punctuation():
    text = (
        "her har hun høstet hvaler\n"
        "eller er en eller annen linje\n"
        "... her.. var ... ellipse...\n"
        "- her - er - bindestrek-\n"
        "? hva med? spørsmålstegn ?\n"
    ).splitlines()

    result = extract_alliteration(text)
    assert len(result) == 2
    assert result[0]["symbol"] == "h"
    assert result[1]["symbol"] == "e"
    assert not any(line["symbol"] in ["...", "-", "?"] for line in result)



import pytest

from poetry_analysis.utils import gather_stanza_annotations


def print_line_number(text: str):
    lines = []
    for i, line in enumerate(text.split("\n"), 1):
        lines.append(f"{i}: {line}")
    return lines


def test_gather_stanza_annotations_splits_double_newlines():
    text = "Stanza 1\nLine 1\nLine 2\n\nStanza 2\nLine 1\nLine 2"
    result_whole_text = print_line_number(text)
    result_stanza_annotations = gather_stanza_annotations(print_line_number)(text)

    assert len(result_whole_text) == 7
    assert result_whole_text == pytest.approx(
        [
            "1: Stanza 1",
            "2: Line 1",
            "3: Line 2",
            "4: ",
            "5: Stanza 2",
            "6: Line 1",
            "7: Line 2",
        ]
    )
    assert len(result_stanza_annotations) == 2
    assert result_stanza_annotations == pytest.approx(
        {
            "stanza_1": ["1: Stanza 1", "2: Line 1", "3: Line 2"],
            "stanza_2": ["1: Stanza 2", "2: Line 1", "3: Line 2"],
        }
    )



import json
import re
import string
from pathlib import Path
from typing import Generator

from convert_pa import convert_nofabet, nofabet_to_ipa

PUNCTUATION_MARKS = str(
    string.punctuation + "‒.,!€«»’”—⁷⁶⁰–‒––!”-?‒"
)  # Note! The three long dashes look identical, but are different unicode characters

VALID_NUCLEI = [
    "aa",
    "ae",
    "oe",
    "ou",
    "ei",
    "øy",
    "ai",
    "oi",
    "au",
    "a",
    "e",
    "i",
    "o",
    "u",
    "y",
    "æ",
    "ø",
    "å",
]

GRAMMATICAL_SUFFIXES = [
    "ene",
    "ane",
    "et",
    "te",
    "er",
    # "ar", # Too common in short words to filter out
    "en",
]


def is_grammatical_suffix(string: str) -> bool:
    return string in GRAMMATICAL_SUFFIXES


def endswith(sequence: str | list[str], suffix: str) -> bool:
    """Check if a sequence ends with a given suffix."""
    if isinstance(sequence, str):
        return sequence.endswith(suffix)
    elif isinstance(sequence, list):
        last_element = sequence.copy().pop()
        if isinstance(last_element, str):
            return last_element.endswith(suffix)
        return False
    return False


def is_punctuation(char: str) -> bool:
    """Check if a character is a punctuation mark."""
    return char in PUNCTUATION_MARKS


def strip_redundant_whitespace(text: str) -> str:
    """Strip redundant whitespace and reduce it to a single space."""
    return re.sub(r"\s+", " ", text).strip()


def strip_punctuation(string: str) -> str:
    """Remove punctuation from a string"""
    alphanumstr = ""
    for char in string:
        if not is_punctuation(char):
            alphanumstr += char
    return strip_redundant_whitespace(alphanumstr)


def make_comparable_string(input: list | str) -> str:
    """Convert a list of strings into a single comparable string."""
    if isinstance(input, list):
        string = " ".join(input)
    else:
        string = str(input)
    string = strip_punctuation(string)
    string = re.sub(r"[0123]", "", string)  # remove stress markers
    return string.casefold()


def convert_to_syllables(phonemes: str | list, ipa: bool = False) -> list:
    """Turn a sequence of phonemes into syllable groups."""
    transcription = phonemes if isinstance(phonemes, str) else " ".join(phonemes)
    if ipa:
        ipa = nofabet_to_ipa(transcription)
        syllables = ipa.split(".")
    else:
        nofabet_syllables = convert_nofabet.nofabet_to_syllables(transcription)
        syllables = [" ".join(syll) for syll in nofabet_syllables]
    return syllables


def syllabify(transcription: list[list]) -> list:
    """Flatten list of syllables from a list of transcribed words."""
    syllables = [
        syll  # if syll is not None else "NONE"
        for word, pron in transcription
        for syll in convert_to_syllables(pron, ipa=False)
    ]
    return syllables


def split_orthographic_text_into_syllables(words: list[str]) -> list:
    """
    WORK IN PROGRESS
    Split orthographic text into syllables using basic rules.
    This is a simplified implementation and may not handle all edge cases.

    Args:
        words (list of str): A list of orthographic words, already tokenized

    Returns:
        list: A list of syllables for each word in the text.
    """
    syllables = []

    for word in words:
        word_syllables = []
        current_syllable = ""

        for i, char in enumerate(word):
            current_syllable += char

            # Check if the character is a vowel
            if char in VALID_NUCLEI:
                # Check if the next character could be part of the same nucleus
                is_not_last = i + 1 < len(word)
                if is_not_last and word[i + 1] in VALID_NUCLEI:
                    continue

                # Check if the next character could be part of a valid onset
                if is_not_last and word[i + 1] not in VALID_NUCLEI:
                    consonant_cluster = char + word[i + 1]
                    if len(consonant_cluster) > 1 and is_valid_onset(consonant_cluster):
                        continue

                # Otherwise, split the syllable
                word_syllables.append(current_syllable)
                current_syllable = ""

        # Add any remaining characters as a syllable
        if current_syllable:
            word_syllables.append(current_syllable)

        syllables.append(word_syllables)

    return syllables


def is_valid_onset(phonelist: str) -> bool:
    """
    WORK IN PROGRESS
    Check if a sequence of characters forms a valid onset in Norwegian orthography.

    Args:
        phonelist (str): A string representing the onset (e.g., "bl", "tr").

    Returns:
        bool: True if the onset is valid, False otherwise.
    """
    # Define valid single consonants and consonant clusters for Norwegian
    valid_single_consonants = set("bcdfghjklmnpqrstvwxyz")
    valid_clusters = {
        "bj",
        "bl",
        "br",
        "dr",
        "dj",
        "fl",
        "fj",
        "fr",
        "gl",
        "gr",
        "gj",
        "kj",
        "kl",
        "kr",
        "kn",
        "kv",
        "pl",
        "pj",
        "pr",
        "mj",
        "nj",
        "sj",
        "sl",
        "sm",
        "sn",
        "sp",
        "st",
        "sv",
        "tr",
        "tj",
        "tl",
        "vr",
        "sk",
        "skr",
        "spr",
        "str",
        "skj",
        "gn",
        "hv",
    }

    if len(phonelist) == 1 and phonelist in valid_single_consonants:
        return True

    if phonelist in valid_clusters:
        return True
    return False


def annotate_transcriptions(transcription: list) -> Generator:
    for word, pronunciation in transcription:
        nofabet = format_transcription(pronunciation)
        yield dict(
            word=word,
            nofabet=nofabet,
            syllables=convert_nofabet.nofabet_to_syllables(nofabet),
            ipa=nofabet_to_ipa(nofabet),
        )


def split_paragraphs(text: str) -> list:
    """Split a text into paragraphs and paragraphs into lines."""
    return [
        [line.rstrip() for line in paragraph.rstrip().splitlines()]
        for paragraph in re.split("\n{2,}", text)
        if paragraph
    ]


def format_transcription(pronunciation):
    return " ".join(pronunciation)


def gather_stanza_annotations(func) -> callable:
    """Decorator to apply a function to each stanza in a text."""

    def wrapper(text: str) -> dict:
        stanzas = split_stanzas(text)
        stanza_annotations = {}
        for i, stanza in enumerate(stanzas, 1):
            stanza_text = "\n".join(stanza)
            stanza_annotations[f"stanza_{i}"] = func(stanza_text)
        return stanza_annotations

    return wrapper


def split_stanzas(text: str) -> list:
    """Split a poem into stanzas and stanzas into verses."""
    return [
        [verse.rstrip() for verse in stanza.rstrip().splitlines()]
        for stanza in re.split("\n{2,}", text)
        if stanza
    ]


def annotate(func, text: str, stanzaic: bool = False, outputfile: str | Path = None):
    if stanzaic:
        new_func = gather_stanza_annotations(func)
        annotations = new_func(text)
    else:
        annotations = func(text)
    if outputfile is not None:
        Path(outputfile).write_text(
            json.dumps(annotations, indent=4, ensure_ascii=False), encoding="utf-8"
        )
        print(f"Saved annotated data to {outputfile}")
    else:
        return annotations


if __name__ == "__main__":
    import doctest

    doctest.testmod()



import pytest

from poetry_analysis import rhyme_detection as rd


@pytest.mark.parametrize(
    "word, expected",
    [
        ("Der", "e"),
        ("var", "a"),
        ("Bælter", "æ"),
        ("af", "a"),
        ("Silke", "i"),
        ("og", "o"),
        ("Ringer", "i"),
        ("af", "a"),
        ("Guld", "u"),
    ],
)
def test_find_correct_orthographic_nucleus(word, expected):
    result = rd.find_nucleus(word, orthographic=True).group(1)

    assert result == expected


@pytest.mark.parametrize(
    "word, expected",
    [
        ("EE1 N", "EE"),
        ("V AH2 N D R AX0 N AX0", "AH"),
        ("S V EH1 N", "EH"),
        ("M EE1", "EE"),
        ("S II1 N", "II"),
        ("KJ OE2 P M AH0 N S B OO3 D", "OE"),
    ],
)
def test_find_correct_phonemic_nucleus(word, expected):
    result = rd.find_nucleus(word, orthographic=False).group(1)

    assert result == expected



"""Anaphora is the repetition of the same line-initial word or phrase
in a verse, or across consecutive verses in a stanza.

TODO: It can also refer to the repetition of a whole stanza-initial verse line
in consecutive stanzas.
> NOTE: This has not been implemented yet.
This anaphora detection process is based on the repetition of the first word in each line. We will continue with implementing a grading system for how effective the figure is in each poem.
"""

from collections import Counter, defaultdict
from pathlib import Path

from poetry_analysis.utils import annotate, split_stanzas, strip_punctuation


def count_initial_phrases(text: str) -> Counter:
    """Count the number of times initial phrases of different lengths occur in a string."""
    phrase_counts = Counter()
    words = strip_punctuation(text).split()
    n_words = len(words)

    for n in range(1, n_words + 1):
        if len(words) >= n:
            phrase = " ".join(words[:n])
            count = text.count(phrase)
            if count > 0:
                phrase_counts[phrase] += count
    return phrase_counts


def find_longest_most_frequent_anaphora(phrases: Counter) -> dict:
    """Find the longest and most repeated word sequence in a line."""
    if not phrases:
        return None, 0

    _, highest_count = phrases.most_common()[0]
    top_phrases = [
        phrase
        for phrase, _ in phrases.most_common()
        if phrases[phrase] == highest_count
    ]

    longest_phrase = max(top_phrases, key=len)
    longest_count = phrases[longest_phrase]

    annotation = (longest_phrase, longest_count)
    return annotation


def extract_line_anaphora(text: str) -> list:
    """Extract line initial word sequences that are repeated at least twice on the same line."""
    anaphora = []
    lines = text.strip().lower().split("\n")
    for i, line in enumerate(lines):
        line_initial_phrases = count_initial_phrases(line)
        phrase, count = find_longest_most_frequent_anaphora(line_initial_phrases)
        if count > 1:
            annotation = {"line_id": i, "phrase": phrase, "count": count}
            anaphora.append(annotation)
    return anaphora


def is_successive(items: list):
    return [items[i] == items[i - 1] + 1 for i, item in enumerate(items)][1:]


def extract_poem_anaphora(text: str) -> list:
    """Extract line-initial word sequences that are repeated at least twice in each stanza."""
    anaphora = []

    stanzas = split_stanzas(text)
    for i, stanza in enumerate(stanzas):
        stanza_anaphora = extract_stanza_anaphora(stanza)
        for phrase, indeces in stanza_anaphora.items():
            if len(indeces) <= 1:
                continue
            if all(is_successive(indeces)):
                annotation = {
                    "stanza_id": i,
                    "line_id": indeces,
                    "phrase": phrase,
                    "count": len(indeces),
                }
                anaphora.append(annotation)
    return anaphora


def extract_stanza_anaphora(stanza: list[str]) -> dict:
    stanza_anaphora = {}

    for line_index, line in enumerate(stanza):
        if not line:
            continue
        first_word = line.split()[0].lower()
        previous_line = stanza[line_index - 1].lower().split()
        try:
            previous_first_word = previous_line[0]
        except IndexError:
            previous_first_word = None

        if line_index > 0 and previous_first_word == first_word:
            stanza_anaphora[first_word].append(line_index)
        else:
            stanza_anaphora[first_word] = [line_index]

    return stanza_anaphora


def detect_repeating_lines(text: str) -> list:
    """Detect repeating lines in a poem."""
    stanzas = split_stanzas(text)
    lines = [line.strip() for stanza in stanzas for line in stanza]

    repeating_lines = {}
    for idx, line in enumerate(lines):
        if line in repeating_lines:
            repeating_lines[line].append(idx)
        else:
            total = lines.count(line)
            if total > 1:
                repeating_lines[line] = [idx]

    return [(indeces, line) for line, indeces in repeating_lines.items()]


def extract_anaphora(text: str) -> dict:
    """Extract line-initial word sequences that are repeated at least twice.

    Example use:
    >>> import json
    >>> text = '''
    ... Jeg ser paa den hvide himmel,
    ... jeg ser paa de graablaa skyer,
    ... jeg ser paa den blodige sol.
    ...
    ... Dette er altsaa verden.
    ... Dette er altsaa klodernes hjem.
    ...
    ... En regndraabe!
    ... '''
    >>> result = extract_anaphora(text)
    >>> print(json.dumps(result, indent=4))
    {
        "1-grams": {
            "jeg": 3,
            "dette": 2
        },
        "2-grams": {
            "jeg ser": 3,
            "dette er": 2
        },
        "3-grams": {
            "jeg ser paa": 3,
            "dette er altsaa": 2
        },
        "4-grams": {
            "jeg ser paa den": 2
        }
    }
    """
    lines = text.strip().lower().split("\n")
    ngram_counts = defaultdict(lambda: defaultdict(int))

    for line in lines:
        text = strip_punctuation(line)
        words = text.split()
        n_words = len(words)
        for n in range(1, n_words + 1):
            if len(words) >= n:
                ngram = " ".join(words[:n])
                ngram_counts[n][ngram] += 1

    anaphora = {}
    for n in range(1, 5):
        ngram_type = f"{n}-grams"
        ngrams = {ngram: count for ngram, count in ngram_counts[n].items() if count > 1}
        if ngrams:
            anaphora[ngram_type] = ngrams
    return anaphora


if __name__ == "__main__":
    import doctest

    doctest.testmod()

    # Parse user arguments
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("textfile", help="Filepath to the text to analyze.")
    parser.add_argument(
        "--split_stanzas", action="store_true", help="Split the text into stanzas."
    )
    args = parser.parse_args()

    # Analyze the text
    filepath = Path(args.textfile)
    text = filepath.read_text()

    output_file = Path(filepath.parent / f"{filepath.stem}_anaphora.json")
    annotate(
        extract_anaphora, text, stanzaic=args.split_stanzas, outputfile=output_file
    )
    print(f"Anaphora saved to file: {output_file}")



import pytest

from poetry_analysis import rhyme_detection as rd


def test_transcriptions_return_correct_rhyme_tags(transcribed_poem_lines):
    # Given
    result = rd.tag_rhyming_verses(transcribed_poem_lines, orthographic=False)
    assert result[0].rhyme_tag == result[1].rhyme_tag
    assert result[2].rhyme_tag == result[3].rhyme_tag


def test_orthographic_verses_return_correct_rhyme_tag():
    # Given

    verses = [
        "Der var Ufred i Landet Paa Torvet stod",
        "En vandrende Svend med sin Kjøbmandsbod",
        "Af skinnende Smykker laa Disken fuld",
        "Der var Bælter af Silke og Ringer af Guld",
    ]
    result = rd.tag_rhyming_verses(verses, orthographic=True)
    assert result[0].rhyme_tag == result[1].rhyme_tag
    assert result[2].rhyme_tag == result[3].rhyme_tag


def test_tag_rhyming_verses_scores_proper_rhymes_more_than_0():
    # Given
    verses = [
        ["L OH2 K AX0 S"],
        ["S K R UH2 M P AX0 R", "IH3 N"],
        ["SJ L OH2 K AX0 S"],
        ["OEH1 R K AX0 N V IH3 N"],
    ]
    result = rd.tag_rhyming_verses(verses, orthographic=False)
    assert result[2].rhyme_score > 0
    assert result[3].rhyme_score > 0



import pytest

from poetry_analysis import rhyme_detection as rd


@pytest.mark.parametrize(
    "word1, word2",
    [
        ("klangen", "klangen"),
        ("tusenfryd", "fryd"),
        ("avisen", "kulturavisen"),
    ],
)
def test_noedrim_scores_half(word1, word2):
    result = rd.score_rhyme(word1, word2, orthographic=True)
    assert result == 0.5


@pytest.mark.parametrize(
    "word1, word2",
    [
        ("klangen", "sangen"),
        ("bjellen", "makrellen"),
        ("syr", "myr"),
        ("hjerte", "smerte"),
    ],
)
def test_orthographic_proper_rhymes(word1, word2):
    result = rd.score_rhyme(word1, word2, orthographic=True)
    assert result == 1.0


@pytest.mark.parametrize(
    "word1, word2",
    [
        ("G UH L", "J UH L"),
        ("G UH L", "F UH L"),
        ("B OO D", "S T OO D"),
    ],
)
def test_phonemic_proper_rhyme(word1, word2):
    result = rd.score_rhyme(word1, word2, orthographic=False)
    assert result == 1.0


@pytest.mark.parametrize(
    "word1, word2",
    [
        ("klangen", "fryd"),
        ("UU2 F R EH3 D", "L AH2 N AX0"),
        ("S V EH1 N", "SJ IH2 N NX0 AX0"),
    ],
)
def test_different_words_score_zero(word1, word2):
    result = rd.score_rhyme(word1, word2)
    assert result == 0.0


@pytest.mark.parametrize(
    "word1, word2",
    [
        ("sang", "seng"),
        ("song", "sing"),
        ("søym", "draum"),
        ("svei", "sved"),
    ],
)
def test_no_common_vowels_scores_zero(word1, word2):
    result = rd.score_rhyme(word1, word2)
    assert result == 0.0


@pytest.mark.parametrize(
    "word1, word2",
    [
        ("sleden", "bilen"),
        ("husene", "byene"),
        # ("spiste", "lyste"), # Can't handle this case yet
        ("gutane", "tankane"),
        ("fester", "blomster"),
        ("diktet", "brevet"),
        ("arbeidet", "jogget"),
    ],
)
def test_common_grammatical_ending_scores_zero(word1, word2):
    result = rd.score_rhyme(word1, word2)
    assert result == 0.0


@pytest.mark.parametrize(
    "word1, word2",
    [
        ("klangen", "klang"),
        ("klang", "klangen"),
        ("arbeider", "arbeidene"),
        ("klangen", "sang"),
        ("sang", "klangen"),
    ],
)
def test_common_substring_in_start_or_middle_scores_zero(word1, word2):
    result = rd.score_rhyme(word1, word2)
    assert result == 0.0



import pytest

from poetry_analysis import rhyme_detection as rt


@pytest.mark.parametrize(
    "syllable",
    [
        "AE1",
        ["H", "EE3", "D"],
        ["H", "EH1", "D"],
        "L IH2 T AX0",
        "K AE2 N",
    ],
)
def test_vowels_with_toneme_or_secondary_stress_markers_are_stressed(syllable):
    """Test that stress markers are correctly identified."""
    # when
    result = rt.is_stressed(syllable)
    # then
    assert result


@pytest.mark.parametrize(
    "vowel",
    [
        "AE",
        "IH",
        "EH",
        "UW",
        "AA",
        "AH",
        "AO",
        "IY",
        "UH",
        "OW",
        "AW",
        "OY",
        "ER",
        "AX",
        "IX",
    ],
)
def test_vowels_without_stress_markers_are_not_stressed(vowel):
    """Test that vowels without stress markers are not stressed."""
    # when
    result = rt.is_stressed(vowel)
    # then
    assert not result


@pytest.mark.parametrize("schwa", ["AX0", "AX0 N", "AX0 N AX0", "AX0 R"])
def test_schwa_is_not_stressed(schwa):
    """Test that schwa is not stressed."""
    # when
    result = rt.is_stressed(schwa)
    # then
    assert not result
